// C.Dupaty - 03/2012
// binliotheque C18 pour MAX7456 sur PIC18F2620

#include "max7456.h"
#include <delays.h>
#include <stdio.h>
#include "max7456.h"

rom const char char_max7456[]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0x48,0,0,0,0,0x46,0x3F,0x40,0,0,0x45,0x49,0x41,0x47,
0xA,1,2,3,4,5,6,7,8,9,0x44,0x43,0x4A,0,0x4B,0x42,0x4C,0xB,0xC,0xD,0xE,0xF,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,
0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x20,0x21,0x22,0x23,0x24,0,0,0,0,0,0x46,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,
0x2D,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E };

unsigned char val_DMAH,val_DMAL;

// tempo ecriture pour MAX7456
void tempo_tCSW(void)
{
//	Delay1KTCYx(1);
	Nop();
}

// tempo 100ms (pour le RESET du MAX7456)
void tempo_100ms(unsigned int t)
{
	do		// 1cycle = 125nS, 100ms=800000 cycles
		{
			Delay10KTCYx(80);
		}
	while(t--);
}


unsigned char emet_spi(unsigned char s)
{unsigned char a;
	a=SSPBUF; // efface BF 
	SSPBUF=s; // emission
	while(!SSPSTATbits.BF);
	return (SSPBUF);
}

unsigned char lit_max7456(unsigned char ad)
{unsigned char r;
	CS=0;
	emet_spi(ad);
// SPI sur front descendant, SCK=0 au repos
	SSPCON1bits.CKP=1;
	SSPSTATbits.CKE=1;
	r=emet_spi(0);
	CS=1;
// SPI sur front descendant, SCK=0 au repos
	SSPCON1bits.CKP=1;
	SSPSTATbits.CKE=0;
	tempo_tCSW();
	return(r);
}

void emet_max7456(unsigned char ad,unsigned char data)
{
	CS=0;
	emet_spi(ad);
	emet_spi(data);
	CS=1;
	tempo_tCSW();
}

// video inverse si C!=0
void inverse(unsigned char c)
{unsigned char n;
	n=lit_max7456(DM_MODE_READ);
	if (c) n|=0b00001000;
	else n&=0b11110111;
	emet_max7456(DM_MODE_WRITE,n);
}


// si C!=0 clignotement
void clignote(unsigned char c)
{unsigned char n;
	n=lit_max7456(DM_MODE_READ);
	if (c) n|=0b00010000;
	else n&=0b11101111;
	emet_max7456(DM_MODE_WRITE,n);
}

// positionne le curseur en x/y (colonne/ligne) (30x16)
void gotoxy(unsigned char x, unsigned char y)
{
	unsigned int n;
	n=30*(unsigned int)y+(unsigned int)x;	// conversion ligne/colonne en adresse
	val_DMAH=n>>8; // PF
	val_DMAL=n&0x00FF; // pf
}

// emet un caractere (code max5456 page 19)
void putc_max7456(unsigned char c)
{
	emet_max7456(DM_ADDRH_WRITE,val_DMAH);//DMAH
	emet_max7456(DM_ADDRL_WRITE,val_DMAL);//DMAL
	emet_max7456(DM_CODE_IN_WRITE,c);// correspondance ASCII - table max7456
// pointe adresse suivante
	if (val_DMAL ==255) {
							val_DMAH++; 
						    val_DMAL=0;
						}					
	else val_DMAL++;
	Nop();
}

// emet un code ACSII 
void _user_putc(char d)
{
	emet_max7456(DM_ADDRH_WRITE,val_DMAH);//DMAH
	emet_max7456(DM_ADDRL_WRITE,val_DMAL);//DMAL
	emet_max7456(DM_CODE_IN_WRITE,char_max7456[d]);// correspondance ASCII - table max7456
// pointe adresse suivante
	if (val_DMAL ==255) {
							val_DMAH++; 
						    val_DMAL=0;
						}					
	else val_DMAL++;
	Nop();
}


void init_max7456(void)
{
unsigned char x; 

	stdout = _H_USER; // printf sort sur _user_putc

	TRIS_LED=0;
	TRIS_BOUTON=1;
	TRIS_CS=0;
	TRIS_DIN=1;
	TRIS_SCLK=0;
	TRIS_DOUT=0;
	TRIS_RST=0;
	SSPSTATbits.SMP=1;		// SPI Mode Maitre
	SSPCON1bits.SSPEN=1; 	// active SPI
// SPI sur front descendant, SCK=0 au repos
	SSPCON1bits.CKP=1;
	SSPSTATbits.CKE=0;
// SCK : FOSC/4 soit 8MHz (MAX7456 : max=10MHz)
	SSPCON1bits.SSPM3=0;
	SSPCON1bits.SSPM2=0;
	SSPCON1bits.SSPM1=0; // 
	SSPCON1bits.SSPM0=0;
	
	CS=1;
	SCLK=0;
	RST=1;
	RST=0;
	RST=1;
	tempo_100ms(1); 	// d apres datasheet max7456
	emet_max7456(VIDEO_MODE_0_WRITE,PAL|OSD_EN);	// PAL , active incrustation
	//Control automatique du niveau de noir, lire et reecrire le parametre
	//Le data sheet n'est pas trs prcis  ce sujet.
	x = lit_max7456(OSDBL_RD);	
	x &= 0xEF;
	emet_max7456(OSDBL_WR,x);
	emet_max7456(DM_MODE_WRITE,0);//DMM : mode 16bits
}